package TRcon;
#
# TRcon Perl Module - execute commands on a remote Half-Life2 server using remote console.
#
#
# Copyright (C) 2005 Tobias Oetzel
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

use strict;
no strict 'vars';

use Sys::Hostname;
use IO::Socket;
use IO::Select;

my $VERSION = "1.00";
my $TIMEOUT = 0.5;

my $SERVERDATA_EXECCOMMAND        = 2;
my $SERVERDATA_AUTH               = 3;
my $SERVERDATA_RESPONSE_VALUE     = 0;
my $SERVERDATA_AUTH_RESPONSE      = 2;
my $REFRESH_SOCKET_COUNTER_LIMIT  = 25;


#
# Constructor
#

sub new
{
  my ($class_name, $server_object) = @_;
  my ($self) = {};
  bless($self, $class_name);
  
  $self->{"rcon_socket"}            = 0;
  $self->{"server_object"}          = $server_object;
  $self->{"auth"}                   = 0;
  $self->{"refresh_socket_counter"} = 0;
  return $self;
}

sub execute
{
  my ($self, $command) = @_;
  my $answer = $self->sendrecv($command, 0);
  if ($answer =~ /bad rcon_password/i) {
    &::printEvent("TRCON", "Bad Password");
  }
  return $answer;
}

sub get_auth_code
{
  my ($self, $id) = @_;
  my $auth = 0;
  
  if ($id == 1024) {
    &::printEvent("TRCON", "Rcon password accepted");
    $auth = 1;
    $self->{"auth"} = 1;
  } elsif( $id == -1) {
    &::printEvent("TRCON", "Rcon password refused");
    $self->{"auth"} = 0;
    $auth           = 0;
  } else {
    &::printEvent("TRCON", "Bad password response id=$id");
    $self->{"auth"} = 0;
    $auth           = 0;
  }
  return $auth;

}


sub sendrecv
{
  my ($self, $msg) = @_;
  
  my $rs_counter = $self->{"refresh_socket_counter"};
  if ($rs_counter % $REFRESH_SOCKET_COUNTER_LIMIT == 0)  {
    if ($self->{"rcon_socket"}) {
      close($self->{"rcon_socket"});
    }
    my $server_object = $self->{"server_object"};
    $self->{"rcon_socket"}   =  IO::Socket::INET->new(
                                      		Proto=>"tcp",
                                            PeerAddr=>$server_object->get("address"), 
                                            PeerPort=>$server_object->get("port"), 
                                    		LocalAddr=>"$s_ip",
                            	) or die ("\nCan't setup TCP socket on $ip$s_port: $!\n");
    $self->{"refresh_socket_counter"} = 0;
    $self->{"auth"} = 0;
  }                          	


  my $r_socket = $self->{"rcon_socket"};
  my $server   = $self->{"server_object"};

  my $auth     = $self->{"auth"};
  my $response = "";
  
  if ($r_socket) {
    if ($auth == 0)  {
      &::printEvent("TRCON", "Trying to get rcon access (auth)");
      if ($self->send_rcon(1024, $SERVERDATA_AUTH, $server->get("rcon"), "")) {
        &::printEvent("TRCON", "Couldn't send password");
        return;
      }
      my ($id, $command, $response) = $self->recieve_rcon();
      if($command == $SERVERDATA_AUTH_RESPONSE) {
        $auth = $self->get_auth_code($id);
      } elsif (($command == $SERVERDATA_RESPONSE_VALUE) && ($id == 1024)) {  
         # CS:Source server sends one junk packet during the authentication step, before it responds 
         # with the correct authentication response.  
         
         &::printEvent("TRCON", "Junk packet from Counterstrike:Source");
         my ($id, $command, $response) = $self->recieve_rcon();
         $auth = $self->get_auth_code($id);
      }  
    }
    
    if ($auth == 1)  {
      # send command
      $self->{"refresh_socket_counter"}++;
      $self->send_rcon(2048, $SERVERDATA_EXECCOMMAND, $msg);
      # retrieve data
      my ($id, $command, $response) = $self->recieve_rcon();
      if(($id == 2048) && ($command == $SERVERDATA_RESPONSE_VALUE)) {
        return $response;
      }
    }
  }  
  return;
  
}

#
# Send a package
#
sub send_rcon
{
  my ($self, $id, $command, $string1, $string2) = @_;
  my $data = pack("VVZ*Z*", $id, $command, $string1, $string2);
  my $size = length($data);
  if($size > 4096) {
    &::printEvent("TRCON", "Command to long to send!");
    return 1;
  }
  my $data = pack("V", $size).$data;

  my $r_socket = $self->{"rcon_socket"};
  $r_socket->send($data, 0);
  
  return 0;
}

sub is_new_paket {
  my ($self, $size, $id) = @_;
  if ((($size > 10) && ($size < 4096)) && (($id == 2048) || ($id == 1024) || ($id == 0))) {
    return 1;
  } else {
    return 0;  
  }  
}

#
#  Recieve a package
#
sub recieve_rcon
{
  my $self = shift;
  my ($size, $id, $command, $msg);
  my $tmp = "";

  my $r_socket = $self->{"rcon_socket"};
  my $server   = $self->{"server_object"};
  my $auth     = $self->{"auth"};
  
  if ($r_socket) {
    if(IO::Select->new($r_socket)->can_read($TIMEOUT)) {  # $TIMEOUT seconds timeout
      $r_socket->recv($tmp, 2000);
      
      $size    = unpack("V",  substr($tmp, 0, 4));
      $id      = unpack("V",  substr($tmp, 4, 4));
      $command = unpack("V",  substr($tmp, 8, 4));
      
      if ($self->is_new_paket($size, $id) == 1)  {
        $tmp     = substr($tmp, 12, length($tmp)-12);
        while (IO::Select->new($r_socket)->can_read(0))  {
          my $data;
          $r_socket->recv($data, 2000);
          $tmp .= $data;
        }
        my ($string1, $string2) = unpack("Z*Z*", $tmp);
        $msg = $string1.$string2;
      }
      return ($id, $command, $msg);
    } else {
      return (-1, -1, -1);
    }
  }
}

#
# Get error message
#

sub error
{
  my ($self) = @_;
  return $self->{"rcon_error"};
}



#
# Parse "status" command output into player information
#

sub getPlayers
{
  my ($self) = @_;
  my $status = $self->execute("status");
  my @lines = split(/[\r\n]+/, $status);

  my %players;


# userid name uniqueid connected ping loss state adr
# 187 ".:[SoV]:.Evil Shadow" STEAM_0:1:6200412 13:48 97 0 active 213.10.196.229:24085

  foreach my $line (@lines)
  {
    if ($line =~ /^\#\s*
                (\d+)\s+		# userid
                "(.+)"\s+		# name
                (.+)\s+		    # uniqueid
                ([\d:]+)\s+		# time
                (\d+)\s+		# ping
                (\d+)\s+		# loss
                (.+)\s+			# state
                ([^:]+):    	# addr
                (\S+)           # port
                $/x)
    {
      my $userid   = $1;
      my $name     = $2;
      my $uniqueid = $3;
      my $time     = $4;
      my $ping     = $5;
      my $loss     = $6;
      my $state    = $7;
      my $address  = $8;
      my $port     = $9;
      #print "USERID: '$userid', NAME: '$name', UNIQUEID: '$uniqueid', TIME: '$time', PING: '$ping', LOSS: '$loss', STATE: '$state', ADDRESS:'$address', CLI_PORT: '$port'\n";

      $players{$uniqueid} = { 
                           "Name"       => $name,
                           "UserID"     => $userid,
                           "UniqueID"   => $uniqueid,
                           "Time"       => $time,
                           "Ping"       => $ping,
                           "Loss"       => $loss,
                           "State"      => $state,
                           "Address"    => $address,
                           "ClientPort" => $port
                          };
    }
  }
  return %players;
}


#
# Get information about a player by userID
#


sub getPlayer
{
  my ($self, $uniqueid) = @_;
  my %players = $self->getPlayers();
  
  if (defined($players{$uniqueid}))
  {
    return $players{$uniqueid};
  }
  else
  {
    $self->{"error"} = "No such player # $uniqueid";
    return 0;
  }
}

sub getServerData
{
  my ($self) = @_;
  my $status = $self->execute("status");
  
  my @lines = split(/[\r\n]+/, $status);
  
  my $map         = "N/A";
  my $max_players = 0;
  foreach my $line (@lines)
  {
    #      build   :  2260
    #      udp/ip  :  213.133.123.103:27015
    #      map     :  de_dust2 at: 0 x, 0 y, 0 z
    #      players :  12 (16 max)
    if ($line =~ /^\s*map\s*:\s*([\S]+).*$/x)
    {
      $map   = $1;
    }
    if ($line =~ /^\s*players\s*:.+\(([\S]+).*$/x)
    {
      $max_players = $1;
    }
  }
  return ($map, $max_players);
}


1;
# end
